/* Copyright 2003 Andrew Cleveland
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#include <stdio.h>
#include <stdlib.h>
#include <errno.h>

#define VERSION "0.1"

struct message {
    char comment[8192];
    char user[8192];
    int vote; //-1 = delete, 0 = no vote, 1 = keep
    struct message *next;
};

void clear()
{
#ifdef _WIN32
  system("cls");
#endif

#ifdef _linux
  system("clear");
#endif
}

const char userstart[] = "[[User:";
const char userend[] = "]]";
const char seperator[] = "|";

char *extractmsg(struct message *msg, char *raw)
{
    msg->comment[0] = '\0';
    msg->user[0] = '\0';
    char *nulladdr;
    char *username = strstr(raw, userstart);
    if(!username) return NULL;
    memcpy(msg->comment, raw, username - raw);
    nulladdr = msg->comment + (username - raw);
    *nulladdr = '\0';
    username += strlen(userstart);
   
    char *endname;
    char *endfullname;
    //look for pipe or double right bracket, whichever comes first
    if(strstr(username, userend) < strstr(username, seperator))
    {
        endname = strstr(username, userend);
        endfullname = endname;
    }
    else if(strstr(username, seperator) < strstr(username, userend))
    {
        endname = strstr(username, seperator);
        endfullname = strstr(username, userend);
    }
    if(!endname) return NULL;
    memcpy(msg->user, username, endname - username);
    nulladdr = msg->user + (endname - username);
    *nulladdr = '\0';
    return (endfullname + strlen(userend) + 1);
}

int iscounted(struct message *chkmsg, struct message *firstmsg)
{
    struct message *curmsg = firstmsg;
    struct message *nextmsg;
    while(1)
    {
        if(curmsg == chkmsg) break;
        if(strcmp(chkmsg->user, curmsg->user) == 0 && curmsg->vote)
          return 1;
        nextmsg = curmsg->next;
        if(!nextmsg) break; //this should not occur
        curmsg = nextmsg;
    }
    return 0;
}

int main(int argc, char *argv[])
{
  printf("Wikivtal v%s\nEvil saltine (imthesponge@hotmail.com)\n", VERSION);
  if(argc < 2)
  {
    fprintf(stderr, "Syntax: wikivtal votes.txt [results.txt]\n");
    return -1;
  }
  FILE *file;
  file = fopen(argv[1], "rb");
  if(!file)
  {
    fprintf(stderr, "Could not open %s: ", argv[1]);
    perror(""); 
    return -1;
  }
  
  // read file
  fseek(file, 0, SEEK_END);
  int size = ftell(file);
  fseek(file, 0, SEEK_SET);
  char *fbuf = (char*)malloc(size);
  int temp = fread(fbuf, 1, size, file);
  if(temp != size)
  {
    printf("%i, %i", temp, size);
    perror("Could not read from file: ");
  }
  fclose(file);
  
  //index comments/votes
  struct message firstmsg;
  struct message *curmsg = &firstmsg;
  struct message *prevmsg;
  char *curpos = fbuf;
  while(1)
  {
    if(!(curpos = extractmsg(curmsg, curpos)))
    {
        free(prevmsg->next);
        prevmsg->next = NULL;
        break;
    }
    prevmsg = curmsg;
    prevmsg->next = (struct message*)malloc(sizeof(struct message));
    curmsg = prevmsg->next;
  }
  printf("Messages indexed.\n");
  
  //print/tabulate comments/votes
  
  int delvotes = 0;
  int keepvotes = 0;
  
  char deleteusers[1024];
  strcpy(deleteusers, "Delete\n------\n");
  char keepusers[1024];
  strcpy(keepusers, "Keep\n------\n");
  
  int i=0;
  curmsg = &firstmsg;
  struct message *nextmsg;
  int goodchar;
  char c;
  while(1)
  {
    clear();
    printf("Vote %i: %s\n::: User:%s\n", i, curmsg->comment, curmsg->user);
    if(!iscounted(curmsg, &firstmsg))
    {
        printf("\nDelete - D, Keep - K, No vote - N, Quit - Q: ");
        goodchar = 0;
        curmsg->vote = 0;
        while(!goodchar)
        {
            c = getch();
            switch(toupper(c))
            {
              case 'D':
                delvotes++;
                curmsg->vote = -1;
                strcat(deleteusers, curmsg->user);
                strcat(deleteusers, "\n");
                goodchar = 1;
                break;
              case 'K':
                keepvotes++;
                curmsg->vote = 1;
                strcat(keepusers, curmsg->user);
                strcat(keepusers, "\n");
                goodchar = 1;
                break;
              case 'N':
                goodchar = 1;
                break;
              case 'Q':
                return 0;
            }
        }
        putchar(c);
        putchar('\n');
    }
    nextmsg = curmsg->next;
    curmsg = nextmsg;
    if(!curmsg) break;
    i++;
  }
  clear();
  printf("First comment:\n\n%s\n\nNominated by %s\n", firstmsg.comment, firstmsg.user);
  if(delvotes + keepvotes == 0) fprintf(stderr, "No votes.\n");
  else
    printf("Votes to delete: %i (%i%%); votes to keep: %i (%i%%)\n",
              delvotes, (int)(100*(float)(delvotes/(float)(delvotes+keepvotes))),
              keepvotes, (int)(100*(float)(keepvotes/(float)(delvotes+keepvotes))));
  printf("Press Enter to see votes:");
  getchar();
  putchar('\n');
  fputs(deleteusers, stdout);
  putchar('\n');
  fputs(keepusers, stdout);
  putchar('\n');
  
  if(argc==3)
  {
    FILE *resfile = fopen(argv[2], "w");
    if(!resfile)
    {
        fprintf(stderr, "Could not open %s: ", argv[3]);
        perror(""); 
        return -1;
    }
    fprintf(resfile, "First comment:\n\n%s\n\nNominated by %s\n", firstmsg.comment, firstmsg.user);
    fprintf(resfile, "Votes to delete: %i (%i%%); votes to keep: %i (%i%%)\n",
              delvotes, (int)(100*(float)(delvotes/(float)(delvotes+keepvotes))),
              keepvotes, (int)(100*(float)(keepvotes/(float)(delvotes+keepvotes))));
    fputc('\n', resfile);
    fputs(deleteusers, resfile);
    fputc('\n', resfile);
    fputs(keepusers, resfile);
    fclose(resfile);
    printf("Results saved to %s.\n", argv[2]);
  }
  return 0;
}
